#include <signal.h>
#include <stdarg.h>
#include <assert.h>
#include <unistd.h>
#include <aalib.h>
#include <vgakeyboard.h>
#include "jp.h"

void jp_init_level();

char *dbg_filename = "log.txt";
t_obj *jp_jetpac;

t_objset *set_fire;
t_objset *set_wkfo;
t_objset *set_all;

void
signal_int(int sig)
{

	printf("konec filma by sig %i\n", sig);
	jp_close();

} /* singal_int */

int
jp_init(int argc, char *argv[])
{
	int i;

	context = NULL;
	has_vgalib = 0;

	unlink(dbg_filename);

	set_fire = objset_new();
	assert(set_fire);

	set_all = objset_new();
	assert(set_all);
	
	set_trava = objset_new();
	assert(set_trava);

	signal(SIGINT, signal_int);
	signal(SIGSEGV, signal_int);
	signal(SIGTERM, signal_int);
	signal(SIGHUP, signal_int);
	signal(SIGINT, signal_int);
	signal(SIGQUIT, signal_int);
	signal(SIGILL, signal_int);
	signal(SIGABRT, signal_int);
	signal(SIGFPE, signal_int);
	signal(SIGSEGV, signal_int);
	signal(SIGPIPE, signal_int);
	signal(SIGALRM, signal_int);
	signal(SIGTERM, signal_int);
	signal(SIGUSR1, signal_int);
	signal(SIGUSR2, signal_int);
	signal(SIGCHLD, signal_int);
	signal(SIGBUS, signal_int);

	jp_jetpac = obj_new(JP_JETPAC);

	DB("jp_init:argc:%i", argc);

	if( (1 == argc) && (-1 !=  keyboard_init()) )
	{
		keytable = keyboard_getstate();
		has_vgalib = 1;
	}
	else
	{
		has_vgalib = 0;
	}
 
	context = aa_autoinit(&aa_defparams);
	/*
	 * aa_autoinitkbd(context, AA_SENDRELEASE);
	 */

	if( ! has_vgalib )
	{
		aa_autoinitkbd(context, 0);
		keytable = keytable_nahradni;
	}
	 


	if(context == NULL) {
		fprintf(stderr, "Cannot initialize AA-lib. Sorry\n");
		exit(1);
	}


	for(i=256;i--;)
	{
		jp_palette[i] = i;
	}

	 /* FIXME:dej to do init.c a do jp.h*/
	jp_screen = aa_image(context);
	jp_screen_w = aa_imgwidth(context);
	jp_screen_h = aa_imgheight(context);
	jp_init_level();

	return 0;
} /* jp_init */

void
jp_close()
{
	if( has_vgalib )
	{
		keyboard_close();
	}

	if(context)
	{	
		aa_close(context);
	}

	exit(-1);

} /* jp_close */


void
jp_dbg(char *p, ...)
{
	va_list ap;
	FILE *f;

	va_start(ap, p);

	f = fopen(dbg_filename, "at");

	if( f )
	{
		vfprintf(f, p, ap);
		fprintf(f, "\n");
		fclose(f);
	}

	va_end(ap);
} /* jp_dbg */

void
jp_init_level()
{
	t_obj *o;
#define OBJ_SIZ(oo, spz, xx, yy, dxx, dyy) \
	{ \
		oo = obj_new(spz);   \
		assert(oo);    \
		(oo)->x = (xx); \
		(oo)->y = (yy); \
		(oo)->dx = dxx; \
		(oo)->dy = dyy; \
	}

#define JP_X_PT(a) ( (jp_screen_w * (a)) / 80 )
#define JP_Y_PT(a) ( (jp_screen_h * (a)) / 25 )

	DB("jp_init_level");

	OBJ_SIZ(o, JP_TRAVA, JP_X_PT(0), JP_Y_PT(23), JP_X_PT(80), JP_Y_PT(2));
	OBJ_SIZ(o, JP_TRAVA, JP_X_PT(0), JP_Y_PT(8), JP_X_PT(15), JP_Y_PT(2));
	OBJ_SIZ(o, JP_TRAVA, JP_X_PT(30), JP_Y_PT(11), JP_X_PT(15), JP_Y_PT(2));
	OBJ_SIZ(o, JP_TRAVA, JP_X_PT(65), JP_Y_PT(5), JP_X_PT(15), JP_Y_PT(2));

} /* init_level */
